<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to a commercial license from PosadzimyPl
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the PosadzimyPl is strictly forbidden.
 * In order to obtain a license, please contact us: posadzimy.pl
 *
 * @author    PosadzimyPl
 * @copyright Copyright (c) 2017-2021 PosadzimyPl
 * @license   Commercial license
 * @package   PosadzimyPlResponseModule
 */

class UserOrders extends ObjectModel
{
    const STATUS_MAIL_ID_NOT_SEND = 1;
    const STATUS_MAIL_ID_SEND = 2;
    const STATUS_BASE_NEW = 1;
    const STATUS_BASE_FAIL = 2;
    const STATUS_BASE_DONE = 3;

    public $id;

    public $id_customer;

    public $id_order;

    public $id_status;

    public $id_mail_status;

    public $id_order_api;

    public $module_order_value;

    public $certificate_url;

    public $certificate_image_url;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'pp_user_orders',
        'primary' => 'id_pp_user_orders',
        'fields' => array(
            'id_customer' =>            array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'id_order' =>               array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'id_status' =>              array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'id_mail_status' =>         array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'id_order_api' =>           array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'module_order_value' =>     array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice'),
            'certificate_url' =>        array('type' => self::TYPE_HTML, 'validate' => 'isCleanHtml'),
            'certificate_image_url' =>  array('type' => self::TYPE_HTML, 'validate' => 'isCleanHtml'),
        ),
    );

    public function __construct($id = null, $lang = null, $shop = null)
    {
        parent::__construct($id, $lang, $shop);
    }

    public static function getUserOrderByOrderId($orderId)
    {
        $dbquery = new DbQuery();
        $dbquery->select('puo.`id_pp_user_orders`');
        $dbquery->from('pp_user_orders', 'puo');
        $dbquery->where('puo.id_order = ' . (int)$orderId);

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($dbquery);

        return empty($result) ? $result : $result['id_pp_user_orders'];
    }

    public static function getAmountValueById($userOrderId)
    {
        $dbquery = new DbQuery();
        $dbquery->select('puo.`module_order_value`');
        $dbquery->from('pp_user_orders', 'puo');
        $dbquery->where('puo.id_pp_user_orders = ' . (int)$userOrderId);

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($dbquery);

        return empty($result) ? 0 : $result['module_order_value'];
    }

    public static function getAllFailUserOrders()
    {
        $dbquery = new DbQuery();
        $dbquery->select('puo.`id_pp_user_orders`');
        $dbquery->from('pp_user_orders', 'puo');
        $dbquery->where('puo.id_status = ' . self::STATUS_BASE_FAIL);

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($dbquery);
    }
}
