<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to a commercial license from PosadzimyPl
 * Use, copy, modification or distribution of this source file without written
 * license agreement from the PosadzimyPl is strictly forbidden.
 * In order to obtain a license, please contact us: posadzimy.pl
 *
 * @author    PosadzimyPl
 * @copyright Copyright (c) 2017-2021 PosadzimyPl
 * @license   Commercial license
 * @package   PosadzimyPlResponseModule
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

include(_PS_MODULE_DIR_.'/posadzimypl/classes/Hooks.php');
include(_PS_MODULE_DIR_.'/posadzimypl/classes/BaseConfiguration.php');
include(_PS_MODULE_DIR_.'/posadzimypl/classes/UserOrders.php');
include(_PS_MODULE_DIR_.'/posadzimypl/classes/ApiError.php');

class PosadzimyPl extends Module
{
    const CONFIG_CONFIGURATION_ID = 'PP_CONFIG_CONFIGURATION_ID';
    const SUBMIT_CONFIGURATION = 'submitConfiguration';
    const SUBMIT_HOOKS = 'submitHooks';

    const TAB_ID_INSTRUCTION = 1;
    const TAB_ID_CONFIGURATION = 2;
    const TAB_ID_HOOKS = 3;

    const API_CHECK_CREDIT_BALANCE = '/api/integration/check_credit_balance';
    const API_MAKE_ORDER = '/api/integration/make_order';
    const API_MAIL_CONTENT = '/api/integration/customer_mail_content';

    const API_URI = 'https://api.posadzimy.pl';

    const ADMIN_PANEL_TABS = array(
        array(
            'class_name' => 'AdminAjax',
            'name' => 'Admin ajax',
            'hide' => true,
        ),
    );

    public $languages;

    public $defaultLanguageId;

    public function __construct()
    {
        $this->name = 'posadzimypl';
        $this->tab = 'market_place';
        $this->version = '1.0.0';
        $this->author = 'PrestaPros.com';
        $this->need_instance = 0;
        $this->languages = Language::getLanguages(false);
        $this->defaultLanguageId = Configuration::get('PS_LANG_DEFAULT');
        $this->ps_versions_compliancy = array(
            'min' => '1.6.0.5',
            'max' => _PS_VERSION_
        );
        $this->bootstrap = true;
        $this->controllers = array(
            'AdminAjaxController',
        );
        parent::__construct();

        $this->displayName = $this->l('Posadzimy.pl');
        $this->description = $this->l('Połącz swój sklep z Posadzimy.pl i dodawaj sadzenie drzew do każdego'
            .' zamówienia, spełniające minimalną wartość. Twoi klienci otrzymają certyfikat posadzenia drzewa,'
            .' a na Twojej stronie w Posadzimy.pl będzie widoczna zbiorcza liczba posadzonych drzew.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');
        if (!parent::install()
            || !$this->registerViewHooks()
            || !$this->registerHook('newOrder')
            || !$this->registerHook('updateOrderStatus')
            || !$this->registerHook('header')
        ) {
            return false;
        }
        $this->registerTabs();
        $this->addHooks();
        Configuration::updateValue(self::CONFIG_CONFIGURATION_ID, $this->addDefaultConfiguration());

        return true;
    }

    public function uninstall()
    {
        include(dirname(__FILE__).'/sql/uninstall.php');

        $this->unRegisterTabs();

        return parent::uninstall();
    }

    public function hookHeader()
    {
        if ($this->checkCreditBalance()) {
            $this->context->controller->addCSS($this->_path . 'views/css/front_posadzimypl.css');
        }
    }

    public function hookDisplayTop($params)
    {
        if ($this->checkCreditBalance()) {
            if (version_compare(_PS_VERSION_, '1.7.0', '<')
                && isset($this->context->smarty->tpl_vars['page_name'])
                && 'order' === $this->context->smarty->tpl_vars['page_name']->value
            ) {
                return $this->renderViewForHookCart($params, 'displayTop');
            }
        }
    }

    public function hookDisplayPosadzimyCustom()
    {
        if ($this->checkCreditBalance()) {
            return $this->renderViewForHook('displayPosadzimyCustom');
        }
    }

    public function hookDisplayCheckoutSubtotalDetails($params)
    {
        if ($this->checkCreditBalance()) {
            return $this->renderViewForHookCart($params, 'displayCheckoutSubtotalDetails');
        }
    }

    public function hookExtraRight($params)
    {
        if ($this->checkCreditBalance()) {
            return $this->renderViewForHookCart($params, 'extraRight');
        }
    }

    public function hookDisplayCheckoutSummaryTop($params)
    {
        if ($this->checkCreditBalance()) {
            return $this->renderViewForHookCart($params, 'displayCheckoutSummaryTop');
        }
    }

    public function hookDisplayProductAdditionalInfo($params)
    {
        if ($this->checkCreditBalance()) {
            return $this->renderViewForHookCart($params, 'displayProductAdditionalInfo');
        }
    }

    public function hookNewOrder($params)
    {
        if ($this->checkCreditBalance()) {
            if (!Validate::isLoadedObject($params['customer']) || !Validate::isLoadedObject($params['order'])) {
                die($this->l('Missing parameters'));
            }
            $cart = new Cart((int)$params['order']->id_cart);
            $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));
            $amountValue = $baseConfiguration->order_value * $this->context->currency->conversion_rate;
            $amountMissing = $amountValue - $cart->getOrderTotal();
            if (0 > $amountMissing) {
                $userOrders = new UserOrders();
                $userOrders->id_customer = (int)$params['customer']->id;
                $userOrders->id_order = (int)$params['order']->id;
                $userOrders->id_status = UserOrders::STATUS_BASE_NEW;
                $userOrders->id_mail_status = UserOrders::STATUS_MAIL_ID_NOT_SEND;
                $userOrders->module_order_value = $amountValue;

                $userOrders->save();
            }
        }
    }

    public function hookUpdateOrderStatus($params)
    {
        if ($this->checkCreditBalance()) {
            if (!Validate::isLoadedObject($params['newOrderStatus'])) {
                $this->context->controller->errors[] = $this->l('Missing order status parameter.');
                return;
            }
            $newOrder = $params['newOrderStatus'];
            $order = new Order((int)$params['id_order']);
            if ($order && !Validate::isLoadedObject($order)) {
                $this->context->controller->errors[] = $this->l('Incorrect Order object.');
                return;
            }
            $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));
            $userOrderId = UserOrders::getUserOrderByOrderId($order->id);
            $amountValue = empty($userOrderId) ?
                $baseConfiguration->order_value * $this->context->currency->conversion_rate :
                UserOrders::getAmountValueById($userOrderId);
            $amountMissing = $amountValue - $order->getTotalProductsWithTaxes();
            if (0 > $amountMissing) {
                $acceptedUserStatuses = explode(',', $baseConfiguration->order_statuses);
                if (in_array($newOrder->id, $acceptedUserStatuses)) {
                    $userOrder = empty($userOrderId) ?
                        $this->addNewUserOrder($order->id_customer, $order->id, $amountValue) :
                        new UserOrders($userOrderId);
                    if (UserOrders::STATUS_BASE_DONE === (int)$userOrder->id_status) {
                        return;
                    }
                    if ($this->createApiOrder($userOrder, $this->context->language->id)) {
                        $this->context->controller->confirmations[] = $this->l('Data sent to api correctly.');
                        $userOrder->id_status = UserOrders::STATUS_BASE_DONE;
                        if ($baseConfiguration->custom_mail_active) {
                            if ($this->prepareMail(
                                $userOrder,
                                $this->context->language->id,
                                $this->context->shop->name
                            )) {
                                $this->context->controller->confirmations[] = $this->l(
                                    'The e-mail with the certificate has been successfully sent to the client.'
                                );
                                $userOrder->id_mail_status = UserOrders::STATUS_MAIL_ID_SEND;
                            } else {
                                $this->context->controller->errors[] = $this->l(
                                    'An error has occurred,'
                                    .' the e-mail with the certificate has not been successfully sent to the client.'
                                );
                            }
                        }
                    } else {
                        $this->context->controller->errors[] = $this->l(
                            'An error has occurred, data has not been sent to api.'
                        );
                        $userOrder->id_status = UserOrders::STATUS_BASE_FAIL;
                    }
                    $userOrder->update();
                }
            }
        }
    }

    public function getContent()
    {
        $activeIdTab = $this->postProcess();
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));

        $this->context->controller->addJS($this->_path . 'views/js/adminMenu.js');
        $this->context->controller->addJS($this->_path . 'views/js/rivets.min.js');
        $this->context->controller->addCSS($this->_path . 'views/css/posadzimypl.css');

        $creditBalance = $this->getCreditBalance(
            self::API_URI,
            $baseConfiguration->api_key,
            $baseConfiguration->tree_group_id
        );

        $this->context->smarty->assign(array(
            'module_name' => $this->name,
            'module_version' => $this->version,
            'module_display' => $this->displayName,
            'configuration_form' => $this->renderForm(
                self::SUBMIT_CONFIGURATION,
                $this->getConfigurationFields(),
                $this->getConfigurationForm($creditBalance)
            ),
            'hooks_form' => $this->renderHooksForm(),
            'ps_version' => (bool)version_compare(_PS_VERSION_, '1.6', '>'),
            'activeTabId' => $activeIdTab,
            'adminAjaxToken' => Tools::getAdminTokenLite('AdminAjax'),
            'credit_balance' => $creditBalance,

        ));

        return $this->display(__FILE__, '/views/templates/admin/configuration.tpl');
    }

    public function postProcess()
    {
        $activeIdTab = self::TAB_ID_INSTRUCTION;
        if (Tools::isSubmit(self::SUBMIT_CONFIGURATION)) {
            $this->updateBaseConfiguration();
            $activeIdTab = self::TAB_ID_CONFIGURATION;
        }
        if (Tools::isSubmit(self::SUBMIT_HOOKS)) {
            $hooksId = Hooks::getAllHooksId();
            foreach ($hooksId as $item) {
                $this->updateHook($item['id_pp_hooks']);
            }
            $activeIdTab = self::TAB_ID_HOOKS;
        }

        return $activeIdTab;
    }

    public function testApiConnection($apiKey, $treeGroupId)
    {
        $url = self::API_URI . self::API_CHECK_CREDIT_BALANCE;
        $response = $this->sendRequest(
            $url,
            'POST',
            $apiKey,
            array('treeGroupID' => $treeGroupId)
        );
        if (200 === (int)$response['code']) {
            return array(
                'code' => $response['code'],
                'messages' => $this->l('Dane poprawne, połączono prawidłowo.')
            );
        }

        return array(
            'code' => $response['code'],
            'messages' => $this->l('Podane dane nie są poprawne, połączenie nie powiodło się.')
        );
    }

    public function checkCreditBalance()
    {
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));
        $creditBalance = $this->getCreditBalance(
            self::API_URI,
            $baseConfiguration->api_key,
            $baseConfiguration->tree_group_id
        );

        return $creditBalance > 0;
    }

    protected function getCreditBalance($apiUri, $apiKey, $treeGroupId)
    {
        $url = $apiUri . self::API_CHECK_CREDIT_BALANCE;
        $data = $this->sendRequest($url, 'POST', $apiKey, array('treeGroupID' => $treeGroupId));
        $creditBalance = 0;
        if (200 === (int)$data['code'] && isset($data['response'])) {
            $response = json_decode($data['response']);
            if (isset($response->data) && isset($response->data->creditsBalance)) {
                $creditBalance = $response->data->creditsBalance;
            }
        }

        return $creditBalance;
    }

    public function againCreateApiOrderForFailOrders()
    {
        if ($this->checkCreditBalance()) {
            $failUserOrders = UserOrders::getAllFailUserOrders();
            $baseConfiguration = new BaseConfiguration(
                (int)Configuration::get(self::CONFIG_CONFIGURATION_ID),
                $this->defaultLanguageId
            );
            if (count($failUserOrders) > 0) {
                foreach ($failUserOrders as $failUserOrder) {
                    $userOrder = new UserOrders((int)$failUserOrder['id_pp_user_orders']);
                    if ($this->createApiOrder($userOrder, $this->defaultLanguageId)) {
                        $userOrder->id_status = UserOrders::STATUS_BASE_DONE;
                        if ($baseConfiguration->custom_mail_active) {
                            if ($this->prepareMail(
                                $userOrder,
                                $this->defaultLanguageId,
                                Configuration::get('PS_SHOP_NAME')
                            )
                            ) {
                                $userOrder->id_mail_status = UserOrders::STATUS_MAIL_ID_SEND;
                            }
                        }
                        $userOrder->update();
                    }
                }
            }

            return true;
        }

        return false;
    }

    protected function createApiOrder($userOrder, $langId)
    {
        $customer = new Customer((int)$userOrder->id_customer);
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID), $langId);
        $url = self::API_URI . self::API_MAKE_ORDER;
        $response = $this->sendRequest(
            $url,
            'POST',
            $baseConfiguration->api_key,
            array(
                'customerName' => $customer->firstname,
                'certificateText' => $baseConfiguration->certificate_text,
                'treeGroupID' => $baseConfiguration->tree_group_id,
            )
        );
        if (201 === (int)$response['code']) {
            $data = json_decode($response['response']);
            $userOrder->id_order_api = (int)$data->data->orderId;
            $userOrder->certificate_url = $data->data->certificateUrl;
            $userOrder->certificate_image_url = $data->data->certificateImageUrl;
            $userOrder->update();

            return true;
        }
        $apiError = new ApiError();
        $apiError->id_pp_user_orders = (int)$userOrder->id;
        $apiError->code = (int)$response['code'];
        $data = json_decode($response['response']);
        $apiError->response = empty($data) ? '' : $data->message;
        $apiError->save();

        return false;
    }

    protected function sendRequest($url, $method, $key, $data = null)
    {
        $headers = array('Content-Type:application/json', 'X-AUTH-TOKEN:' . $key);
        $curl = curl_init($url);
        if (!empty($data)) {
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
        }
        if ('POST' === $method) {
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "POST");
        } else {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'GET');
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        return array('response' => $response, 'code' => $httpCode);
    }

    protected function prepareMail($userOrder, $langId, $shopName)
    {
        $customer = new Customer((int)$userOrder->id_customer);
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID), $langId);
        $url = self::API_URI . self::API_MAIL_CONTENT;
        $response = $this->sendRequest(
            $url,
            'POST',
            $baseConfiguration->api_key,
            array(
                'customerName' => $customer->firstname,
                'customText' => $baseConfiguration->mail_text,
                'shopName' => $shopName,
                'linkToCertificate' => $userOrder->certificate_url,
                'certificateImageUrl' => $userOrder->certificate_image_url,
                'plantName' => 'example',
            )
        );
        if (200 === (int)$response['code']) {
            $data = json_decode($response['response']);
            $customer = new Customer((int)$userOrder->id_customer);
            if (file_exists(dirname(__FILE__) . '/mails/pl/posadzimy.html')) {
                file_put_contents(dirname(__FILE__) . '/mails/pl/posadzimy.html', $data->data->mailHTMLContent);
            }
            if (file_exists(dirname(__FILE__) . '/mails/en/posadzimy.html')) {
                file_put_contents(dirname(__FILE__) . '/mails/en/posadzimy.html', $data->data->mailHTMLContent);
            }

            return $this->sendMail(
                'posadzimy',
                $baseConfiguration->mail_subject,
                array(),
                $customer->email
            );
        }
        $apiError = new ApiError();
        $apiError->id_pp_user_orders = (int)$userOrder->id;
        $apiError->code = (int)$response['code'];
        $data = json_decode($response['response']);
        $apiError->response = empty($data) ? '' : $data->message;
        $apiError->save();

        return false;
    }

    protected function sendMail(
        $template,
        $subject,
        $templateVars,
        $to
    ) {
        if (version_compare(_PS_VERSION_, '1.7.0', '<')) {
            $dirname = dirname(__FILE__).'/mails/';
        } else {
            $dirname = dirname(__FILE__) . '/../mails/';
        }

        return Mail::Send(
            $this->context->language->id,
            $template,
            $subject,
            $templateVars,
            $to,
            null,
            null,
            null,
            null,
            null,
            $dirname,
            false,
            null,
            null,
            null
        );
    }


    protected function addNewUserOrder($customerId, $orderId, $amountValue)
    {
        $userOrder = new UserOrders();
        $userOrder->id_customer = (int)$customerId;
        $userOrder->id_order = (int)$orderId;
        $userOrder->id_status = UserOrders::STATUS_BASE_NEW;
        $userOrder->id_mail_status = UserOrders::STATUS_MAIL_ID_NOT_SEND;
        $userOrder->module_order_value = $amountValue;

        $userOrder->save();

        return $userOrder;
    }

    protected function renderViewForHook($name)
    {
        $hook = Hooks::getHookByName($name, $this->context->language->id);
        if (!empty($hook) && 1 === (int)$hook['active']) {
            $this->context->smarty->assign(array(
                'content' => $hook['content'],
                'text_color' => $hook['text_color'],
                'background_color' => $hook['background_color'],
            ));

            return $this->display(__FILE__, '/views/templates/hook/hook_default.tpl');
        }
    }

    protected function renderViewForHookCart($params, $name)
    {
        $hook = Hooks::getHookByName($name, $this->context->language->id);
        if (!empty($hook) && 1 === (int)$hook['active']) {
            $cartTotal = 0;
            if (Validate::isLoadedObject($params['cart'])) {
                $cartTotal = $params['cart']->getOrderTotal();
            }
            $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));
            $amountValue = $baseConfiguration->order_value * $this->context->currency->conversion_rate;
            $amountMissing = $amountValue - $cartTotal;
            $content = str_replace(
                '{value}',
                Tools::displayPrice($amountMissing, $this->context->currency),
                $hook['content']
            );

            $this->context->smarty->assign(array(
                'content' => $content,
                'text_color' => $hook['text_color'],
                'background_color' => $hook['background_color'],
                'amountDone' => 0 > $amountMissing,
                'isContentAfter' => $hook['after'],
                'content_after' => $hook['content_after'],
                'extendedClass' => $this->getExtendedClassForHook($name),
                'font_size' => $hook['font_size'],
                'custom_css' => $hook['custom_css'],
            ));

            return $this->display(__FILE__, '/views/templates/hook/hook_cart.tpl');
        }
    }

    protected function getExtendedClassForHook($name)
    {
        switch ($name) {
            case 'displayProductAdditionalInfo':
                return 'posadzimy_info_box_product';
            case 'displayCheckoutSubtotalDetails':
                return 'posadzimy_info_box_cart';
            case 'displayCheckoutSummaryTop':
                return 'posadzimy_info_box_summary';
            default:
                return '';
        }
    }

    protected function updateBaseConfiguration()
    {
        $langFields = array('certificate_text', 'mail_text', 'mail_subject');
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));
        foreach ($baseConfiguration as $field => $value) {
            if ('id' === $field) {
                continue;
            }
            if ('order_statuses' === $field) {
                $baseConfiguration->$field = !empty(Tools::getValue($field)) ?
                    implode(',', Tools::getValue($field)) :
                    $value;
                continue;
            }
            if (in_array($field, $langFields)) {
                $values = array();
                foreach ($this->languages as $language) {
                    $values[$language['id_lang']] = Tools::getValue($field . '_' . $language['id_lang']);
                }
                $baseConfiguration->$field = $values;
                continue;
            }
            $baseConfiguration->$field = !empty(Tools::getValue($field)) ? Tools::getValue($field) : $value;
        }
        try {
            $baseConfiguration->update();
        } catch (Exception $exception) {
            $this->context->controller->errors[] = str_replace('BaseConfiguration->', '', $exception->getMessage());

            return false;
        }
        $this->context->controller->confirmations[] = $this->l('Configuration updated successfully');

        return true;
    }

    protected function updateHook($hookId)
    {
        $hook = new Hooks($hookId);
        foreach ($hook as $field => $value) {
            if ('id' === $field) {
                continue;
            }
            if ('content' === $field || 'content_after' === $field) {
                $content = array();
                foreach ($this->languages as $language) {
                    $content[$language['id_lang']] = Tools::getValue(
                        $field . '_' . $hookId . '_' . $language['id_lang']
                    );
                }
                $hook->$field = $content;
                continue;
            }
            if ('active' === $field || 'font_size' === $field) {
                $hook->$field = (int)Tools::getValue($field . '_' . $hookId);
                continue;
            }
            $hook->$field = !empty(Tools::getValue($field . '_' . $hookId)) ?
                Tools::getValue($field . '_' . $hookId) :
                $value;
        }
        try {
            $hook->update();
        } catch (Exception $exception) {
            $this->context->controller->errors[] = str_replace('Hooks->', '', $exception->getMessage());

            return false;
        }

        return true;
    }

    protected function registerViewHooks()
    {
        $oldVersion = version_compare(_PS_VERSION_, '1.7.0', '<');
        foreach (Hooks::HOOKS_DATA as $item) {
            if (!$item['install']
                || ($oldVersion && '1.7' === $item['version'])
                || (!$oldVersion && '1.6' === $item['version'])
            ) {
                continue;
            }
            if (!$this->registerHook($item['name'])) {
                return false;
            }
        }

        return true;
    }

    protected function addDefaultConfiguration()
    {
        Db::getInstance()->execute(
            'INSERT INTO `'._DB_PREFIX_.'pp_base_configuration` (`id_pp_base_configuration`) VALUE (1)'
        );

        $langValues = array();
        foreach ($this->languages as $language) {
            $langValues[] = '(1, ' . $language['id_lang'] . ')';
        }
        $langValues = implode(',', $langValues);
        Db::getInstance()->execute(
            'INSERT INTO `'._DB_PREFIX_.
            'pp_base_configuration_lang` (`id_pp_base_configuration`, `id_lang`) VALUES ' . $langValues
        );

        return 1;
    }

    protected function addHooks()
    {
        $oldVersion = version_compare(_PS_VERSION_, '1.7.0', '<');
        foreach (Hooks::HOOKS_DATA as $item) {
            if (($oldVersion && '1.7' === $item['version']) || (!$oldVersion && '1.6' === $item['version'])) {
                continue;
            }
            $hook = new Hooks();
            $hook->active = 0;
            $hook->name = $item['name'];
            $hook->background_color = '#ffffff';
            $hook->text_color = '#000000';
            $hook->custom = $item['custom'];
            $hook->variable = $item['variable'];
            $hook->after = $item['content_after'];
            $hook->font_size = 16;
            $hook->add();
        }
    }

    protected function renderForm($submitForm, $fieldsValue, $form)
    {
        $helper = new HelperForm();
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->submit_action = $submitForm;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name;
        $helper->default_form_language = $this->context->language->id;

        $helper->tpl_vars = array(
            'fields_value' => $fieldsValue,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => Configuration::get('PS_LANG_DEFAULT'),
        );

        return $helper->generateForm(array($form));
    }

    protected function getConfigurationFields()
    {
        $baseConfiguration = new BaseConfiguration((int)Configuration::get(self::CONFIG_CONFIGURATION_ID));

        return array(
            'api_key' => $baseConfiguration->api_key,
            'order_value' => $baseConfiguration->order_value,
            'order_statuses[]' => explode(',', $baseConfiguration->order_statuses),
            'certificate_text' => $baseConfiguration->certificate_text,
            'mail_text' => $baseConfiguration->mail_text,
            'tree_group_id' => $baseConfiguration->tree_group_id,
            'mail_subject' => $baseConfiguration->mail_subject,
            'custom_mail_active' => $baseConfiguration->custom_mail_active,
        );
    }

    protected function getConfigurationForm($creditBalance)
    {
        $suggestedOrderStates = $this->getSuggestedOrderStates($this->context->language->id);
        $suggestedOrderStates = array_map(function ($item) {
            return $item['name'];
        }, $suggestedOrderStates);

        return array(
            'form' => array(
                'input' => array(
                    array(
                        'type' => 'html',
                        'name' => 'html_cron',
                        'html_content' => '<hr>Cron link: ' . '<strong>' . $this->context->link->getBaseLink()
                            . 'modules/posadzimypl/cron.php?token='
                            . Tools::substr(Tools::encrypt('posadzimypl/cron'), 0, 10)
                            .'</strong><hr>',
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_head',
                        'html_content' => '<h2>' . $this->l('Konfiguracja połączenia z API') . '</h2>',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Klucz API'),
                        'name' => 'api_key',
                        'col' => 5,
                        'required' => true,
                        'desc' => $this->l('ID sadzenia znajduje się w panelu klienta, w zakładce “Moduł ecommerce”. Więcej informacji ')
                            . '<a class="active-instruction-tab">' . $this->l('tutaj') . '</a>',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('ID sadzenia'),
                        'name' => 'tree_group_id',
                        'col' => 5,
                        'required' => true,
                        'desc' => $this->l('ID sadzenia znajduje się w panelu klienta, w zakładce “Moduł ecommerce”. Więcej informacji ')
                            . '<a class="active-instruction-tab">' . $this->l('tutaj') . '</a>',
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_data',
                        'html_content' => "<button id='api-check-button' type='button' class='btn btn-default'>"
                            . $this->l('Sprawdź poprawność danych API') . '</button><span class="credit-balance">'
                            . $this->l('Liczba kredytów: ') . $creditBalance . '</span>',
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_head',
                        'html_content' => '<h2>' . $this->l('Ustawienia ogólne modułu') . '</h2>',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Wysokość zamówienia powyżej którego sadzone będzie drzewo'),
                        'name' => 'order_value',
                        'col' => 5,
                        'prefix' => $this->context->currency->iso_code,
                        'required' => true,
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Statusy przy których sadzone będzie drzewo'),
                        'name' => 'order_statuses',
                        'class' => 'chosen',
                        'col' => 5,
                        'required' => true,
                        'multiple' => true,
                        'options' => array(
                            'query' => OrderState::getOrderStates($this->context->language->id),
                            'id' => 'id_order_state',
                            'name' => 'name'
                        ),
                        'desc' => $this->l('Przy wyborze kilku statusów, sadzenie odbędzie się tylko przy'
                                .' pierwszej zmianie na jeden z nich. Sugerowane statusy zamówień: ')
                            . implode(', ', $suggestedOrderStates),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Tekst na certyfikacie'),
                        'name' => 'certificate_text',
                        'lang' => true,
                        'placeholder' => $this->l('np. W ramach podziękowania za zakupy w naszym sklepie, posadziliśmy drzewo!'),
                        'desc' => '<span class="open-modal" data-modal-class="modal-image-certificate">'
                            . $this->l('Pokaż przykładowy certyfikat') . '</span>'
                            .'<div class="modal-image-certificate modal-image">'
                            .'<span class="modal-close close cursor" data-modal-class="modal-image-certificate">&times;'
                            .'</span><div class="modal-content-image">'
                            .'<img class="modal-img" src="https://static.posadzimy.pl/integrations/prestashop/certyfikat.jpg">'
                            .'</div></div>',
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'html_head',
                        'html_content' => '<h2>' . $this->l('Ustawiania wysyłki osobnego maila z certyfikatem') . '</h2>',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Wysyłka aktywna'),
                        'name' => 'custom_mail_active',
                        'values' => array(
                            array(
                                'id' => 'type_switch_on',
                                'value' => 1
                            ),
                            array(
                                'id' => 'type_switch_off',
                                'value' => 0
                            )
                        )
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Temat wiadomości'),
                        'name' => 'mail_subject',
                        'lang' => true,
                        'placeholder' => $this->l('np. Drzewo w podziękowaniu za zakupy w naszym sklepie.'),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Dodatkowa informacja w e-mailu'),
                        'name' => 'mail_text',
                        'lang' => true,
                        'class' => 'mail_text_textarea',
                        'desc' => '<span class="open-modal" data-modal-class="modal-image-mail_text">'
                            . $this->l('Pokaż przykładową wiadomość email') . '</span>'
                            .'<div class="modal-image-mail_text modal-image">'
                            .'<span class="modal-close close cursor" data-modal-class="modal-image-mail_text">&times;'
                            .'</span><div class="modal-content-image">'
                            .'<img class="modal-img" src="https://static.posadzimy.pl/integrations/prestashop/email.jpg">'
                            .'</div></div>',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save')
                ),
            ),
        );
    }

    protected function renderHooksForm()
    {
        $hooksId = Hooks::getAllHooksId();
        $fields = array();
        $inputs = array();
        $inputs[] = array(
            'type' => 'html',
            'name' => 'html_head',
            'html_content' => '<h2>' . $this->l('Moduły informacyjne o sadzeniu drzew') . '</h2>',
        );
        $inputs[] = array(
            'type' => 'html',
            'name' => 'html_info',
            'html_content' => '<p>' . $this->l('W tym miejscu możesz wybrać i skonfigurować sekcje informacje'
                    .' które będą się wyświetlać w zależności od tego, czy klient spełnił minimalny próg zamówienia,'
                    .' czy nie. Możesz tutaj poinformować swoich klientów o sadzeniu drzew za zamówienia od'
                    .' odpowiedniej wartości. Jeśli moduł Posadzimy.pl nie jest prawidłowo połączony z Twoim kontem,'
                    .' lub nie posiadasz kredytów, informacje te nie będą się wyświetlać.') . '</p>',
        );
        foreach ($hooksId as $item) {
            $this->getHookFields((int)$item['id_pp_hooks'], $fields);
            $this->getHookInputs((int)$item['id_pp_hooks'], $inputs);
        }

        return $this->renderForm(
            self::SUBMIT_HOOKS,
            $fields,
            array(
                'form' => array(
                    'input' => $inputs,
                ),
            )
        );
    }

    protected function getHookFields($hookId, &$fields)
    {
        $hook = new Hooks($hookId);
        $fields['active_' . $hookId] = $hook->active;
        $fields['name_' . $hookId] = $hook->name;
        $fields['text_color_' . $hookId] = $hook->text_color;
        $fields['background_color_' . $hookId] = $hook->background_color;
        $fields['content_' . $hookId] = $hook->content;
        $fields['content_after_' . $hookId] = $hook->content_after;
        $fields['font_size_' . $hookId] = $hook->font_size;
        $fields['custom_css_' . $hookId] = $hook->custom_css;
    }

    protected function getHookInputs($hookId, &$inputs)
    {
        $hook = new Hooks($hookId);
        if ($hook->after) {
            $content = array(
                'type' => 'textarea',
                'label' => $this->l('Informacja przed osiągnięciem minimalnego progu zamówienia'),
                'name' => 'content_' . $hookId,
                'autoload_rte' => true,
                'lang' => true,
                'class' => 'rte',
            );
            $contentAfter = array(
                'type' => 'textarea',
                'label' => $this->l('Informacja po osiągnięciu minimalnego progu zamówienia'),
                'name' => 'content_after_' . $hookId,
                'autoload_rte' => true,
                'lang' => true,
                'class' => 'rte',
            );
            if (1 === (int)$hook->variable) {
                $content['desc'] = $this->l('Aby wyświetlić brakującą kwotę, użyj zmiennej {value}, np.'
                    .' Za każde zamówienie powyżej 100 zł, posadzimy w Twoim imieniu drzewo. Brakuje Ci jeszcze'
                    .' {value} zł!');
            }
        } else {
            $content = array(
                'type' => 'textarea',
                'label' => $this->l('Informacja przed osiągnięciem minimalnego progu zamówienia'),
                'name' => 'content_' . $hookId,
                'autoload_rte' => true,
                'lang' => true,
                'class' => 'rte',
            );
            if (1 === (int)$hook->variable) {
                $content['desc'] = $this->l('Aby wyświetlić brakującą kwotę, użyj zmiennej {value}, np.'
                    .' Za każde zamówienie powyżej 100 zł, posadzimy w Twoim imieniu drzewo. Brakuje Ci jeszcze'
                    .' {value} zł!');
            }
        }
        switch ($hook->name) {
            case 'displayPosadzimyCustom':
                $name = $this->l('Dodatkowa informacja w dowolnym miejscu strony');
                break;
            case 'displayCheckoutSubtotalDetails':
                $name = $this->l('Ekran po dodaniu produktu do koszyka, strona koszyka');
                break;
            case 'displayCheckoutSummaryTop':
                $name = $this->l('Realizacja zamówienia');
                break;
            case 'displayTop':
                $name = $this->l('Strona koszyka i podsumowania zamówienia');
                break;
            case 'displayProductAdditionalInfo':
            case 'extraRight':
                $name = $this->l('Strona pojedynczego produktu');
                break;
            default:
                $name = $hook->name;
        }

        $inputs[] = array(
            'type' => 'html',
            'name' => 'header',
            'html_content' => "<h1>" . $name . "</h1>",
        );
        $inputs[] = array(
            'type' => 'switch',
            'label' => $this->l('Informacja aktywna'),
            'name' => 'active_' . $hookId,
            'values' => array(
                array(
                    'id' => 'type_switch_on',
                    'value' => 1
                ),
                array(
                    'id' => 'type_switch_off',
                    'value' => 0
                )
            )
        );
        if ($hook->after) {
            $inputs[] = $content;
            $inputs[] = $contentAfter;
        } else {
            $inputs[] = $content;
        }
        $inputs[] = array(
            'type' => 'textarea',
            'label' => $this->l('Własny kod css'),
            'name' => 'custom_css_' . $hookId,
            'col' => 5,
        );
        $inputs[] = array(
            'type' => 'color',
            'label' => $this->l('Tło sekcji'),
            'name' => 'background_color_' . $hookId
        );
        $inputs[] = array(
            'type' => 'color',
            'label' => $this->l('Kolor tekstu'),
            'name' => 'text_color_' . $hookId
        );
        $inputs[] = array(
            'type' => 'text',
            'label' => $this->l('Wielkość Czcionki'),
            'prefix' => 'PX',
            'name' => 'font_size_' . $hookId,
            'col' => 3,
        );
        if (1 === (int)$hook->custom) {
            $inputs[] = array(
                'type' => 'html',
                'name' => 'html_data',
                'html_content' =>
                    "<p>Custom Hook</p><span>Aby wyświetlić to powiadomienie w dowolnym miejscu, dodaj kod,"
                    ." {hook h='" . $hook->name . "'}<span/>",
            );
        }
        $inputs[] = array(
            'type' => 'html',
            'name' => 'html_data',
            'html_content' => '<button type="submit" value="1" id="configuration_form_submit_btn_1" name="'
                . self::SUBMIT_HOOKS . '" class="btn btn-default pull-right"><i class="process-icon-save"></i> '
                . $this->l('Save') . '</button>',
        );
        $inputs[] = array(
            'type' => 'html',
            'name' => 'button',
            'html_content' => '<div style="border-bottom: 1px solid #000; margin-left: -25%; width: 125%;"></div>',
        );
    }

    protected function registerTabs()
    {
        foreach (self::ADMIN_PANEL_TABS as $tabData) {
            $tab = new Tab();
            $tab->class_name = $tabData['class_name'];
            $tab->id_parent = $tabData['hide'] ? -1 : (int)Tab::getIdFromClassName("IMPROVE");
            $tab->module = $this->name;
            foreach ($this->languages as $l) {
                $tab->name[$l['id_lang']] = $this->l($tabData['name']);
            }
            $tab->save();
        }

        return true;
    }

    protected function unRegisterTabs()
    {
        foreach (self::ADMIN_PANEL_TABS as $tabData) {
            $tabId = Tab::getIdFromClassName($tabData['class_name']);
            if (isset($tabId) && !empty($tabId)) {
                $tab = new Tab($tabId);
                $tab->delete();
            }
        }

        return true;
    }

    protected function getSuggestedOrderStates($langId)
    {
        $dbquery = new DbQuery();
        $dbquery->select('ol.`name`');
        $dbquery->from('order_state', 'o');
        $dbquery->join('LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` ol ON ol.`id_order_state` = o.`id_order_state`');
        $dbquery->where('o.`paid` = 1');
        $dbquery->where('o.`delivery` = 0');
        $dbquery->where('ol.`id_lang` = ' . (int)$langId);

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($dbquery);
    }
}
